from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                      DEFAULT_SOURCE_VIEW_NAME, Licenses,
                                      Tasks)

_DATASETNAME = "wikimatrix"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

# ilo min sun are actually not available
_LANGUAGES = ["ilo", "min", "jav", "sun", "ceb", "ind", "tgl", "vie"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{schwenk-etal-2021-wikimatrix,
    title = "{W}iki{M}atrix: Mining 135{M} Parallel Sentences in 1620 Language Pairs from {W}ikipedia",
    author = "Schwenk, Holger  and
      Chaudhary, Vishrav  and
      Sun, Shuo  and
      Gong, Hongyu  and
      Guzm{\'a}n, Francisco",
    editor = "Merlo, Paola  and
      Tiedemann, Jorg  and
      Tsarfaty, Reut",
    booktitle = "Proceedings of the 16th Conference of the European Chapter of the Association for Computational Linguistics: Main Volume",
    month = apr,
    year = "2021",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.eacl-main.115",
    doi = "10.18653/v1/2021.eacl-main.115",
    pages = "1351--1361",
    abstract = "We present an approach based on multilingual sentence embeddings to automatically extract parallel sentences from the content
    of Wikipedia articles in 96 languages, including several dialects or low-resource languages. We do not limit the extraction process to
    alignments with English, but we systematically consider all possible language pairs. In total, we are able to extract 135M parallel sentences
    for 16720 different language pairs, out of which only 34M are aligned with English. This corpus is freely available. To get an indication
    on the quality of the extracted bitexts, we train neural MT baseline systems on the mined data only for 1886 languages pairs, and evaluate
    them on the TED corpus, achieving strong BLEU scores for many language pairs. The WikiMatrix bitexts seem to be particularly interesting
    to train MT systems between distant languages without the need to pivot through English.",
}
"""

_DESCRIPTION = """\
WikiMatrix is automatically extracted parallel sentences from the content of Wikipedia articles in 96 languages, including several dialects
or low-resource languages. 8 languages among them are spoken in Southeast Asia region. In total, there are 135M parallel sentences from 1620
different language pairs.
"""

_HOMEPAGE = "https://github.com/facebookresearch/LASER/tree/main/tasks/WikiMatrix"

_LICENSE = Licenses.CC_BY_SA_4_0.value

_URLs = "https://dl.fbaipublicfiles.com/laser/WikiMatrix/v1/WikiMatrix.{lang1}-{lang2}.tsv.gz"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

config = {
    "jv": ["en", "es", "fr", "id", "it", "pt"],
    "ceb": ["bg", "ar", "ca", "cs", "de", "en", "es", "fi", "fr", "hu", "it", "ja", "nl", "no", "pl", "pt", "ro", "ru", "sv", "uk"],
    "id": [
        "jv",
        "is",
        "it",
        "ja",
        "ko",
        "lt",
        "mk",
        "ml",
        "mr",
        "ne",
        "nl",
        "no",
        "pl",
        "pt",
        "ro",
        "ru",
        "sh",
        "si",
        "sk",
        "sl",
        "sq",
        "sr",
        "sv",
        "sw",
        "ta",
        "te",
        "tl",
        "tr",
        "tt",
        "uk",
        "vi",
        "zh",
        "ar",
        "az",
        "ba",
        "bg",
        "bn",
        "bs",
        "ca",
        "cs",
        "da",
        "de",
        "el",
        "en",
        "eo",
        "es",
        "et",
        "eu",
        "fa",
        "fi",
        "fr",
        "gl",
        "he",
        "hi",
        "hr",
        "hu",
    ],
    "tl": ["ar", "bg", "bs", "ca", "cs", "da", "de", "el", "en", "eo", "es", "et", "fi", "fr", "gl", "he", "hr", "hu", "id", "it", "ja", "lt", "mk", "nl", "no", "pl", "pt", "ro", "ru", "sh", "sk", "sl", "sq", "sr", "sv", "tr", "uk", "vi", "zh"],
    "vi": [
        "ar",
        "az",
        "bg",
        "bn",
        "bs",
        "ca",
        "cs",
        "da",
        "de",
        "el",
        "en",
        "eo",
        "es",
        "et",
        "eu",
        "fa",
        "fi",
        "fr",
        "gl",
        "he",
        "hi",
        "hr",
        "hu",
        "id",
        "is",
        "it",
        "ja",
        "ko",
        "lt",
        "mk",
        "ml",
        "mr",
        "nl",
        "no",
        "pl",
        "pt",
        "ro",
        "ru",
        "sh",
        "si",
        "sk",
        "sl",
        "sq",
        "sr",
        "sv",
        "sw",
        "ta",
        "te",
        "tl",
        "tr",
        "uk",
        "zh",
    ],
}
_SUBSETS = set()
for lang, pairs in config.items():
    for pair in pairs:
        _SUBSETS.add("{}-{}".format(lang, pair) if lang < pair else "{}-{}".format(pair, lang))
_SUBSETS = list(_SUBSETS)


class WikiMatrixDataset(datasets.GeneratorBasedBuilder):
    """WikiMatrix is automatically extracted parallel sentences from the content of Wikipedia articles in 96 languages, including several dialects
    or low-resource languages."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"wikimatrix_{subset.replace('-', '_')}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="WikiMatrix source schema",
            schema="source",
            subset_id=f"wikimatrix_{subset.replace('-', '_')}",
        )
        for subset in _SUBSETS
    ] + [
        SEACrowdConfig(
            name=f"wikimatrix_{subset.replace('-', '_')}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description="WikiMatrix Nusantara schema",
            schema="seacrowd_t2t",
            subset_id=f"wikimatrix_{subset.replace('-', '_')}",
        )
        for subset in _SUBSETS
    ]

    DEFAULT_CONFIG_NAME = "wikimatrix_en_id_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text_1": datasets.Value("string"),
                    "text_2": datasets.Value("string"),
                    "text_1_name": datasets.Value("string"),
                    "text_2_name": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        lang1, lang2 = self.config.name.split("_")[1], self.config.name.split("_")[2]
        filepath = Path(dl_manager.download_and_extract(_URLs.format(lang1=lang1, lang2=lang2)))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": filepath},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        with open(filepath, "r") as f:
            data = f.readlines()

        lang1, lang2 = self.config.name.split("_")[1], self.config.name.split("_")[2]
        if self.config.schema == "source":
            for _id, line in enumerate(data):
                line = line.strip().split("\t")
                ex = {
                    "id": str(_id),
                    "text_1": line[1],
                    "text_2": line[2],
                    "text_1_name": lang1,
                    "text_2_name": lang2,
                }
                yield _id, ex

        elif self.config.schema == "seacrowd_t2t":
            for _id, line in enumerate(data):
                line = line.strip().split("\t")
                ex = {
                    "id": str(_id),
                    "text_1": line[1],
                    "text_2": line[2],
                    "text_1_name": lang1,
                    "text_2_name": lang2,
                }
                yield _id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

# Keep blank; dataset has no associated paper
_CITATION = """\
@article{,
  author    = {},
  title     = {},
  journal   = {},
  volume    = {},
  year      = {},
  url       = {},
  doi       = {},
  biburl    = {},
  bibsource = {}
}
"""

_LOCAL = False
_LANGUAGES = ["mya", "ceb", "gor", "hil", "ilo", "ind", "jav", "khm", "lao", "zlm", "nia", "tgl", "tha", "vie"]

_DATASETNAME = "mozilla_pontoon"
_DESCRIPTION = """
This dataset contains crowdsource translations of more than 200 languages for
different Mozilla open-source projects from Mozilla's Pontoon localization platform.
Source sentences are in English.
"""

_HOMEPAGE = "https://huggingface.co/datasets/ayymen/Pontoon-Translations"
_LICENSE = Licenses.BSD_3_CLAUSE.value
_URL = "https://huggingface.co/datasets/ayymen/Pontoon-Translations"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class MozillaPontoonDataset(datasets.GeneratorBasedBuilder):
    """Dataset of translations from Mozilla's Pontoon platform."""

    # Two-letter ISO code is used when available
    # otherwise 3-letter one is used
    LANG_CODE_MAPPER = {"mya": "my", "ceb": "ceb", "gor": "gor", "hil": "hil", "ilo": "ilo", "ind": "id", "jav": "jv", "khm": "km", "lao": "lo", "zlm": "ms", "nia": "nia", "tgl": "tl", "tha": "th", "vie": "vi"}

    # Config to load individual datasets per language
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_eng_{lang}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema for {lang} language",
            schema="source",
            subset_id=f"{_DATASETNAME}_eng_{lang}",
        )
        for lang in _LANGUAGES
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_eng_{lang}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema for {lang} language",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}_eng_{lang}",
        )
        for lang in _LANGUAGES
    ]

    # Config to load all datasets
    BUILDER_CONFIGS.extend(
        [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"{_DATASETNAME} source schema for all languages",
                schema="source",
                subset_id=_DATASETNAME,
            ),
            SEACrowdConfig(
                name=f"{_DATASETNAME}_seacrowd_t2t",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"{_DATASETNAME} SEACrowd schema for all languages",
                schema="seacrowd_t2t",
                subset_id=_DATASETNAME,
            ),
        ]
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "source_string": datasets.Value("string"),
                    "target_string": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # dl_manager not used since dataloader uses HF 'load_dataset'
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"split": "train"},
            ),
        ]

    def _load_hf_data_from_remote(self, language: str) -> datasets.DatasetDict:
        """Load dataset from HuggingFace."""
        hf_lang_code = self.LANG_CODE_MAPPER[language]
        hf_remote_ref = "/".join(_URL.split("/")[-2:])
        return datasets.load_dataset(hf_remote_ref, f"en-{hf_lang_code}", split="train")

    def _generate_examples(self, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        languages = []
        pontoon_datasets = []

        lang = self.config.subset_id.split("_")[-1]
        if lang in _LANGUAGES:
            languages.append(lang)
            pontoon_datasets.append(self._load_hf_data_from_remote(lang))
        else:
            for lang in _LANGUAGES:
                languages.append(lang)
                pontoon_datasets.append(self._load_hf_data_from_remote(lang))

        index = 0
        for lang, lang_subset in zip(languages, pontoon_datasets):
            for row in lang_subset:
                if self.config.schema == "source":
                    example = row

                elif self.config.schema == "seacrowd_t2t":
                    example = {
                        "id": str(index),
                        "text_1": row["source_string"],
                        "text_2": row["target_string"],
                        "text_1_name": "eng",
                        "text_2_name": lang,
                    }
                yield index, example
                index += 1



"""
SNLI Indo is derived from the SNLI corpus, where the premise and hypothesis sentences are translated directly from English to Indonesian using the Google Cloud Translation API. The SNLI corpus is divided into three sets, namely train, development, and test set. The translation process is applied to all the premise and hypothesis sentences in all the three sets. This ensures that the number of sentence pairs obtained is the same as the original SNLI dataset, namely 570k sentence pairs. A filtering process is carried out to remove incomplete sentence pairs and those with a gold label `-`. As a result, 569,027 sentence pairs are obtained.
"""

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import jsonlines

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{suwija2023snli,
  author       = "Suwija Putra, I Made 
        and Siahaan, Daniel 
        and Saikhu, Ahmad",
  title        = "SNLI Indo: A recognizing textual entailment dataset in Indonesian derived from the Stanford Natural Language Inference dataset"
  year         = "2024",
  journal      = "Data in Brief",
  volume       = "52",
  pages        = "109998",
  publisher    = "Elsevier",
  doi          = "https://doi.org/10.1016/j.dib.2023.109998",
  url          = "https://www.sciencedirect.com/science/article/pii/S2352340923010284",
}
"""

_DATASETNAME = "snli_indo"

_DESCRIPTION = """\
The SNLI Indo dataset is derived from the SNLI corpus by translating each premise and hypothesis sentence from English to Indonesia via the Google Cloud Translation API. Premise sentences are crawled image captions from Flickr, and hypothesis sentences are manually created through crowdsourcing. Five annotators are assigned per sentence pair to label the inference relationship as entailment (true), contradiction (false) or neutral (undetermined).
"""

_HOMEPAGE = "https://data.mendeley.com/datasets/k4tjhzs2gd/1"

_LANGUAGES = ["ind"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "train": "https://data.mendeley.com/public-files/datasets/k4tjhzs2gd/files/ee45b2bb-e2ea-47b7-bec4-b6653c467d27/file_downloaded",
        "val": "https://data.mendeley.com/public-files/datasets/k4tjhzs2gd/files/5e47db3c-ea84-4c73-9a2f-bfd57b4e2c05/file_downloaded",
        "test": "https://data.mendeley.com/public-files/datasets/k4tjhzs2gd/files/23aff85c-ff72-48b6-aba1-c1dd5dac216b/file_downloaded",
    }
}

_SUPPORTED_TASKS = [Tasks.TEXTUAL_ENTAILMENT]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class SNLIDataset(datasets.GeneratorBasedBuilder):
    """SNLI Indo is derived from the SNLI corpus, where the premise and hypothesis sentences are translated directly from English to Indonesian using the Google Cloud Translation API. This dataset contains ~570k annotated sentence pairs."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="snli_indo_source",
            version=SOURCE_VERSION,
            description="SNLI Indo source schema",
            schema="source",
            subset_id="snli_indo",
        ),
        SEACrowdConfig(
            name="snli_indo_seacrowd_pairs",
            version=SEACROWD_VERSION,
            description="SNLI Indo SEACrowd schema",
            schema="seacrowd_pairs",
            subset_id="snli_indo",
        ),
    ]

    DEFAULT_CONFIG_NAME = "snli_source"
    labels = ["kontradiksi", "keterlibatan", "netral"] # ["contradiction", "entailment", "neutral" ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "premise": datasets.Value("string"),
                    "hypothesis": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=self.labels),
                }
            )

        elif self.config.schema == "seacrowd_pairs":
            features = schemas.pairs_features(self.labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["val"],
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            with jsonlines.open(filepath) as f:
                i = -1
                for example in f.iter():
                    i += 1
                    yield str(i), {
                        "premise": example["kalimat1"],
                        "hypothesis": example["kalimat2"],
                        "label": example["label emas"],
                    }

        elif self.config.schema == "seacrowd_pairs":
            with jsonlines.open(filepath) as f:
                i = -1
                for example in f.iter():
                    i += 1
                    yield str(i), {
                        "id": str(i),
                        "text_1": example["kalimat1"],
                        "text_2": example["kalimat2"],
                        "label": example["label emas"],
                    }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
import re
import zipfile
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{zhang2023m3exam,
      title={M3Exam: A Multilingual, Multimodal, Multilevel Benchmark for Examining Large Language Models},
      author={Wenxuan Zhang and Sharifah Mahani Aljunied and Chang Gao and Yew Ken Chia and Lidong Bing},
      year={2023},
      eprint={2306.05179},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DATASETNAME = "m3exam"

_DESCRIPTION = """\
M3Exam is a novel benchmark sourced from real and official human exam questions for evaluating LLMs\
in a multilingual, multimodal, and multilevel context. In total, M3Exam contains 12,317 questions in 9\
diverse languages with three educational levels, where about 23% of the questions require processing images\
for successful solving. M3Exam dataset covers 3 languages spoken in Southeast Asia.
"""

_HOMEPAGE = "https://github.com/DAMO-NLP-SG/M3Exam"

_LANGUAGES = ["jav", "tha", "vie"]
_LANG_MAPPER = {"jav": "javanese", "tha": "thai", "vie": "vietnamese"}
_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_LOCAL = False
_PASSWORD = "12317".encode("utf-8")  # password to unzip dataset after downloading
_URLS = {
    _DATASETNAME: "https://drive.usercontent.google.com/download?id=1eREETRklmXJLXrNPTyHxQ3RFdPhq_Nes&authuser=0&confirm=t",
}

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING, Tasks.VISUAL_QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class M3ExamDataset(datasets.GeneratorBasedBuilder):
    """
    M3Exam is a novel benchmark sourced from real and official human exam questions for evaluating LLMs
    in a multilingual, multimodal, and multilevel context. In total, M3Exam contains 12,317 questions in 9
    diverse languages with three educational levels, where about 23% of the questions require processing images
    for successful solving. M3Exam dataset covers 3 languages spoken in Southeast Asia.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [SEACrowdConfig(name=f"{_DATASETNAME}_{lang}_source", version=datasets.Version(_SOURCE_VERSION), description=f"{_DATASETNAME} source schema", schema="source", subset_id=f"{_DATASETNAME}") for lang in _LANGUAGES]
        + [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang}_seacrowd_qa",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"{_DATASETNAME} SEACrowd schema",
                schema="seacrowd_qa",
                subset_id=f"{_DATASETNAME}",
            )
            for lang in _LANGUAGES
        ]
        + [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang}_seacrowd_imqa",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"{_DATASETNAME} SEACrowd schema",
                schema="seacrowd_imqa",
                subset_id=f"{_DATASETNAME}",
            )
            for lang in _LANGUAGES
        ]
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_jav_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "question_text": datasets.Value("string"),
                    "background_description": datasets.Sequence(datasets.Value("string")),
                    "answer_text": datasets.Value("string"),
                    "options": datasets.Sequence(datasets.Value("string")),
                    "language": datasets.Value("string"),
                    "level": datasets.Value("string"),
                    "subject": datasets.Value("string"),
                    "subject_category": datasets.Value("string"),
                    "year": datasets.Value("string"),
                    "need_image": datasets.Value("string"),
                    "image_paths": datasets.Sequence(datasets.Value("string")),
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
            features["meta"] = {
                "background_description": datasets.Sequence(datasets.Value("string")),
                "level": datasets.Value("string"),
                "subject": datasets.Value("string"),
                "subject_category": datasets.Value("string"),
                "year": datasets.Value("string"),
            }
        elif self.config.schema == "seacrowd_imqa":
            features = schemas.imqa_features
            features["meta"] = {
                "background_description": datasets.Sequence(datasets.Value("string")),
                "level": datasets.Value("string"),
                "subject": datasets.Value("string"),
                "subject_category": datasets.Value("string"),
                "year": datasets.Value("string"),
            }

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        lang = self.config.name.split("_")[1]

        data_dir = dl_manager.download(urls)

        if not os.path.exists(data_dir + "_extracted"):
            if not os.path.exists(data_dir + ".zip"):
                os.rename(data_dir, data_dir + ".zip")
            with zipfile.ZipFile(data_dir + ".zip", "r") as zip_ref:
                zip_ref.extractall(data_dir + "_extracted", pwd=_PASSWORD)  # unzipping with password
        if not os.path.exists(data_dir):
            os.rename(data_dir + ".zip", data_dir)
        image_generator = [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir + "_extracted", "data/multimodal-question"),
                    "split": "train",
                },
            ),
        ]

        text_generator = [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir + "_extracted", f"data/text-question/{_LANG_MAPPER[lang]}-questions-test.json"),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir + "_extracted", f"data/text-question/{_LANG_MAPPER[lang]}-questions-dev.json"),
                    "split": "dev",
                },
            ),
        ]
        if "imqa" in self.config.name:
            return image_generator
        else:
            if "source" in self.config.name:
                image_generator.extend(text_generator)
                return image_generator
            else:
                return text_generator

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        lang = self.config.name.split("_")[1]
        if self.config.schema == "source":
            if split == "train":
                filepath_json = os.path.join(filepath, f"{_LANG_MAPPER[lang]}-questions-image.json")
                with open(filepath_json, "r") as file:
                    data = json.load(file)
                idx = 0
                for json_obj in data:
                    image_paths = []
                    for text in [json_obj["question_text"]] + json_obj["options"] + json_obj["background_description"]:
                        matches = re.findall(r"\[image-(\d+)\.(jpg|png)\]", text)
                        if matches:
                            image_path = [os.path.join(filepath, f"images-{_LANG_MAPPER[lang]}/image-{image_number[0]}.{image_number[1]}") for image_number in matches]
                            image_paths.extend(image_path)
                    example = {
                        "question_text": json_obj["question_text"],
                        "background_description": json_obj["background_description"] if "background_description" in json_obj.keys() else None,
                        "answer_text": json_obj["answer_text"],
                        "options": json_obj["options"],
                        "language": json_obj["language"] if "language" in json_obj.keys() else None,
                        "level": json_obj["level"] if "level" in json_obj.keys() else None,
                        "subject": json_obj["subject"] if "subject" in json_obj.keys() else None,
                        "subject_category": json_obj["subject_category"] if "subject_category" in json_obj.keys() else None,
                        "year": json_obj["year"] if "year" in json_obj.keys() else None,
                        "need_image": "yes",
                        "image_paths": image_paths,
                    }
                    yield idx, example
                    idx += 1
            else:
                with open(filepath, "r") as file:
                    data = json.load(file)
                idx = 0
                for json_obj in data:
                    example = {
                        "question_text": json_obj["question_text"],
                        "background_description": json_obj["background_description"] if "background_description" in json_obj.keys() else None,
                        "answer_text": json_obj["answer_text"],
                        "options": json_obj["options"],
                        "language": json_obj["language"] if "language" in json_obj.keys() else None,
                        "level": json_obj["level"] if "level" in json_obj.keys() else None,
                        "subject": json_obj["subject"] if "subject" in json_obj.keys() else None,
                        "subject_category": json_obj["subject_category"] if "subject_category" in json_obj.keys() else None,
                        "year": json_obj["year"] if "year" in json_obj.keys() else None,
                        "need_image": "no",
                        "image_paths": None,
                    }
                    yield idx, example
                    idx += 1

        elif self.config.schema == "seacrowd_qa":
            with open(filepath, "r") as file:
                data = json.load(file)
            idx = 0
            for json_obj in data:
                example = {
                    "id": idx,
                    "question_id": idx,
                    "document_id": idx,
                    "question": json_obj["question_text"],
                    "type": "multiple_choice",
                    "choices": [". ".join(answer.split(". ")[1:]) for answer in json_obj["options"]],
                    "context": "",
                    "answer": [". ".join(answer.split(". ")[1:]) for answer in json_obj["options"] if json_obj["answer_text"] == answer[0]],
                    "meta": {
                        "background_description": json_obj["background_description"] if "background_description" in json_obj.keys() else None,
                        "level": json_obj["level"] if "level" in json_obj.keys() else None,
                        "subject": json_obj["subject"] if "subject" in json_obj.keys() else None,
                        "subject_category": json_obj["subject_category"] if "subject_category" in json_obj.keys() else None,
                        "year": json_obj["year"] if "year" in json_obj.keys() else None,
                    },
                }
                yield idx, example
                idx += 1

        elif self.config.schema == "seacrowd_imqa":
            filepath_json = os.path.join(filepath, f"{_LANG_MAPPER[lang]}-questions-image.json")
            with open(filepath_json, "r") as file:
                data = json.load(file)
            idx = 0
            for json_obj in data:
                image_paths = []
                for text in [json_obj["question_text"]] + json_obj["options"] + json_obj["background_description"]:
                    matches = re.findall(r"\[image-(\d+)\.(jpg|png)\]", text)
                    if matches:
                        image_path = [os.path.join(filepath, f"images-{_LANG_MAPPER[lang]}/image-{image_number[0]}.{image_number[1]}") for image_number in matches]
                        image_paths.extend(image_path)

                example = {
                    "id": idx,
                    "question_id": idx,
                    "document_id": idx,
                    "questions": [json_obj["question_text"]],
                    "type": "multiple_choice",
                    "choices": [". ".join(answer.split(". ")[1:]) for answer in json_obj["options"]],
                    "context": "",
                    "answer": [". ".join(answer.split(". ")[1:]) for answer in json_obj["options"] if json_obj["answer_text"] == answer[0]],
                    "image_paths": image_paths,
                    "meta": {
                        "background_description": json_obj["background_description"] if "background_description" in json_obj.keys() else None,
                        "level": json_obj["level"] if "level" in json_obj.keys() else None,
                        "subject": json_obj["subject"] if "subject" in json_obj.keys() else None,
                        "subject_category": json_obj["subject_category"] if "subject_category" in json_obj.keys() else None,
                        "year": json_obj["year"] if "year" in json_obj.keys() else None,
                    },
                }
                yield idx, example
                idx += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{buechel-etal-2020-learning-evaluating,
    title = "Learning and Evaluating Emotion Lexicons for 91 Languages",
    author = {Buechel, Sven  and
      R{\"u}cker, Susanna  and
      Hahn, Udo},
    editor = "Jurafsky, Dan  and
      Chai, Joyce  and
      Schluter, Natalie  and
      Tetreault, Joel",
    booktitle = "Proceedings of the 58th Annual Meeting of the Association for Computational Linguistics",
    month = jul,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2020.acl-main.112",
    doi = "10.18653/v1/2020.acl-main.112",
    pages = "1202--1217",
}
"""

_DATASETNAME = "memolon"

_DESCRIPTION = """\
MEmoLon is an emotion lexicons for 91 languages, each one covers eight emotional variables and comprises over 100k word entries. There are several versions of the lexicons, the difference being the choice of the expansion model.
"""

_HOMEPAGE = "https://zenodo.org/record/3756607/files/MTL_grouped.zip?download=1"

_LICENSE = Licenses.MIT.value

_URLS = {
    _DATASETNAME: "https://zenodo.org/record/3756607/files/MTL_grouped.zip?download=1",
}

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_LANGUAGES = ["ceb", "tgl", "ind", "sun", "jav", "zsm", "vie", "tha", "mya"]

_LANGUAGE_MAP = {"ceb": "Cebuano", "tgl": "Tagalog", "ind": "Indonesian", "sun": "Sundanese", "jav": "Javanese", "zsm": "Malay", "vie": "Vietnamese", "tha": "Thai", "mya": "Burmese"}

_SUPPORTED_TASKS = [Tasks.EMOTION_CLASSIFICATION]

_LOCAL = False


def seacrowd_config_constructor(lang: str, schema: str, version: str) -> SEACrowdConfig:
    if lang not in _LANGUAGE_MAP:
        raise ValueError(f"Invalid lang {lang}")

    if schema != "source" and schema != "seacrowd_text_multi":
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="memolon_{lang}_{schema}".format(lang=lang, schema=schema),
        version=datasets.Version(version),
        description="MEmoLon {schema} schema for {lang} language".format(lang=_LANGUAGE_MAP[lang], schema=schema),
        schema=schema,
        subset_id="memolon",
    )


class Memolon(datasets.GeneratorBasedBuilder):
    """MEmoLon is an emotion lexicons for 91 languages, each one covers eight emotional variables and comprises over 100k word entries."""

    BUILDER_CONFIGS = [SEACrowdConfig(name=f"{_DATASETNAME}_{lang}_source", version=datasets.Version(_SOURCE_VERSION), description=f"MEmoLon source schema for {lang} language", schema="source", subset_id="memolon") for lang in _LANGUAGE_MAP]

    DEFAULT_CONFIG_NAME = None

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "word": datasets.Value("string"),
                    "valence": datasets.Value("float32"),
                    "arousal": datasets.Value("float32"),
                    "dominance": datasets.Value("float32"),
                    "joy": datasets.Value("float32"),
                    "anger": datasets.Value("float32"),
                    "sadness": datasets.Value("float32"),
                    "fear": datasets.Value("float32"),
                    "disgust": datasets.Value("float32"),
                }
            )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        base_path = Path(dl_manager.download_and_extract(urls))
        lang = self.config.name.split("_")[1]
        train_data_path = base_path / f"{lang}.tsv"

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_data_path,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        rows = []
        with open(filepath, encoding='utf-8') as file:
            for line in file:
                rows.append(line.split("\t"))

        if self.config.schema == "source":
            for key, row in enumerate(rows[1:]):
                example = {"word": row[0], "valence": row[1], "arousal": row[2], "dominance": row[3], "joy": row[4], "anger": row[5], "sadness": row[6], "fear": row[7], "disgust": row[8]}
                yield key, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
The GATITOS (Google's Additional Translations Into Tail-languages: Often Short) dataset is a high-quality, multi-way parallel dataset of tokens and short phrases.
This dataset consists in 4,000 English segments (4,500 tokens) that have been translated into each of 173 languages, 170 of which are low-resource, 23 are spoken in Southeast Asia.
This dataset contains primarily short segments: 93% single tokens, and only 23 sentences (0.6%) have over 5 tokens.
As such it is best thought of as a multilingual lexicon, rather than a parallel training corpus.
The source text is frequent words in the English Language, along with some common phrases and short sentences.
Care has been taken to ensure that they include good coverage of numbers, months, days of the week, swadesh words, and names of the languages themselves (including the endonym).
"""
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{jones2023bilex,
      title={Bilex Rx: Lexical Data Augmentation for Massively Multilingual Machine Translation},
      author={Alex Jones and Isaac Caswell and Ishank Saxena and Orhan Firat},
      year={2023},
      eprint={2303.15265},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
    }
}
"""

_DATASETNAME = "gatitos"

_DESCRIPTION = """\
The GATITOS (Google's Additional Translations Into Tail-languages: Often Short) dataset is a high-quality, multi-way parallel dataset of tokens and short phrases.
This dataset consists in 4,000 English segments (4,500 tokens) that have been translated into each of 173 languages, 170 of which are low-resource, 23 are spoken in Southeast Asia.
This dataset contains primarily short segments: 93% single tokens, and only 23 sentences (0.6%) have over 5 tokens.
As such it is best thought of as a multilingual lexicon, rather than a parallel training corpus.
The source text is frequent words in the English Language, along with some common phrases and short sentences.
Care has been taken to ensure that they include good coverage of numbers, months, days of the week, swadesh words, and names of the languages themselves (including the endonym).
"""

_HOMEPAGE = "https://github.com/google-research/url-nlp/blob/main/gatitos/README.md"

_LANGUAGES = ["ace", "ban", "bbc", "bew", "bjn", "bts", "btx", "bug", "cnh", "hil", "iba", "ilo", "kac", "lus", "mad", "mak", "meo", "min", "pag", "pam", "shn", "tet", "war"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLs = "https://raw.githubusercontent.com/google-research/url-nlp/main/gatitos/{src}_{tgt}.tsv"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class GATITOSDataset(datasets.GeneratorBasedBuilder):
    """The GATITOS (Google's Additional Translations Into Tail-languages: Often Short) dataset is a high-quality, multi-way parallel dataset of tokens and short phrases."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{src_lang}_{tgt_lang}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}_{src_lang}_{tgt_lang}",
        )
        for (src_lang, tgt_lang) in [("eng", lang) for lang in _LANGUAGES] + [(lang, "eng") for lang in _LANGUAGES]
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{src_lang}_{tgt_lang}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}_{src_lang}_{tgt_lang}",
        )
        for (src_lang, tgt_lang) in [("eng", lang) for lang in _LANGUAGES] + [(lang, "eng") for lang in _LANGUAGES]
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "src_text": datasets.Value("string"), "tgt_text": datasets.Value("string")})

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        _, src_lang, tgt_lang = self.config.subset_id.split("_")

        filepath = dl_manager.download_and_extract(_URLs.format(src=src_lang.replace("eng", "en"), tgt=tgt_lang.replace("eng", "en")))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": filepath, "src_lang": src_lang, "tgt_lang": tgt_lang},
            )
        ]

    def _generate_examples(self, src_lang: str, tgt_lang: str, filepath: Path) -> Tuple[int, Dict]:
        if self.config.schema == "source":
            for row_id, row in enumerate(open(filepath)):
                src_text, tgt_text = row.strip().split("\t")
                yield row_id, {"id": row_id, "src_text": src_text, "tgt_text": tgt_text}

        elif self.config.schema == "seacrowd_t2t":
            for row_id, row in enumerate(open(filepath)):
                src_text, tgt_text = row.strip().split("\t")
                yield row_id, {"id": row_id, "text_1": src_text, "text_2": tgt_text, "text_1_name": src_lang, "text_2_name": tgt_lang}



"""
Speech Classification Schema for Multilabel
"""
import datasets

def features(label_names = ["Yes", "No"]):
    return datasets.Features(
        {
            "id": datasets.Value("string"),
            "path": datasets.Value("string"),
            "audio": datasets.Audio(sampling_rate=16_000),
            "speaker_id": datasets.Value("string"),
            "labels": datasets.Sequence(datasets.ClassLabel(names=label_names)),
            "metadata": {
                "speaker_age": datasets.Value("int64"),
                "speaker_gender": datasets.Value("string"),
            }
        }
    )



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{luong-vu-2016-non,
    title = "A non-expert {K}aldi recipe for {V}ietnamese Speech Recognition System",
    author = "Luong, Hieu-Thi  and
      Vu, Hai-Quan",
    editor = "Murakami, Yohei  and
      Lin, Donghui  and
      Ide, Nancy  and
      Pustejovsky, James",
    booktitle = "Proceedings of the Third International Workshop on Worldwide Language Service
    Infrastructure and Second Workshop on Open Infrastructures and Analysis Frameworks for
    Human Language Technologies ({WLSI}/{OIAF}4{HLT}2016)",
    month = dec,
    year = "2016",
    address = "Osaka, Japan",
    publisher = "The COLING 2016 Organizing Committee",
    url = "https://aclanthology.org/W16-5207",
    pages = "51--55",
    abstract = "In this paper we describe a non-expert setup for Vietnamese speech recognition
    system using Kaldi toolkit. We collected a speech corpus over fifteen hours from about fifty
    Vietnamese native speakers and using it to test the feasibility of our setup. The essential
    linguistic components for the Automatic Speech Recognition (ASR) system was prepared basing
    on the written form of the language instead of expertise knowledge on linguistic and phonology
    as commonly seen in rich resource languages like English. The modeling of tones by integrating
    them into the phoneme and using the phonetic decision tree is also discussed. Experimental
    results showed this setup for ASR systems does yield competitive results while still have
    potentials for further improvements.",
}
"""

_DATASETNAME = "vivos"

_DESCRIPTION = """\
VIVOS is a Vietnamese speech corpus consisting of 15 hours of recording speech prepared for
Automatic Speech Recognition task. This speech corpus is collected by recording speech data
from more than 50 native Vietnamese volunteers.
"""

_HOMEPAGE = "https://zenodo.org/records/7068130"

_LANGUAGES = ["vie"]

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False

_URLS = {
    "audio": "https://huggingface.co/datasets/vivos/resolve/main/data/vivos.tar.gz",
    "train_prompt": "https://huggingface.co/datasets/vivos/resolve/main/data/prompts-train.txt.gz",
    "test_prompt": "https://huggingface.co/datasets/vivos/resolve/main/data/prompts-test.txt.gz",
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

logger = datasets.logging.get_logger(__name__)


class VIVOSDataset(datasets.GeneratorBasedBuilder):
    """
    VIVOS is a Vietnamese speech corpus from https://zenodo.org/records/7068130.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_sptext",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_sptext",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "sentence": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """
        Returns SplitGenerators.
        """

        audio_path = dl_manager.download(_URLS["audio"])
        train_prompt_path = dl_manager.download_and_extract(_URLS["train_prompt"])
        test_prompt_path = dl_manager.download_and_extract(_URLS["test_prompt"])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "prompts_path": train_prompt_path,
                    "clips_path": "vivos/train/waves",
                    "audio_files": dl_manager.iter_archive(audio_path),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "prompts_path": test_prompt_path,
                    "clips_path": "vivos/test/waves",
                    "audio_files": dl_manager.iter_archive(audio_path),
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, prompts_path: Path, clips_path: Path, audio_files, split: str) -> Tuple[int, Dict]:
        """
        Yields examples as (key, example) tuples.
        """
        examples = {}
        with open(prompts_path, encoding="utf-8") as f:
            if self.config.schema == "source":
                for row in f:
                    data = row.strip().split(" ", 1)
                    speaker_id = data[0].split("_")[0]
                    audio_path = "/".join([clips_path, speaker_id, data[0] + ".wav"])
                    examples[audio_path] = {
                        "speaker_id": speaker_id,
                        "path": audio_path,
                        "sentence": data[1],
                    }
            elif self.config.schema == "seacrowd_sptext":
                audio_id = 0
                for row in f:
                    data = row.strip().split(" ", 1)
                    speaker_id = data[0].split("_")[0]
                    audio_path = "/".join([clips_path, speaker_id, data[0] + ".wav"])
                    examples[audio_path] = {
                        "id": audio_id,
                        "path": audio_path,
                        "text": data[1],
                        "speaker_id": speaker_id,
                        "metadata": {
                            "speaker_age": None,
                            "speaker_gender": None,
                        },
                    }
                    audio_id += 1

        idx = 0
        for path, f in audio_files:
            if path.startswith(clips_path):
                if path in examples:
                    audio = {"path": path, "bytes": f.read()}
                    yield idx, {**examples[path], "audio": audio}
                    idx += 1
            else:
                continue


# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

# no bibtex citation
_CITATION = ""
_DATASETNAME = "asr_stidusc"
_DESCRIPTION = """\
This open-source dataset consists of 4.56 hours of transcribed Thai scripted
speech focusing on daily use sentences, where 5,431 utterances contributed by
ten speakers were contained.
"""

_HOMEPAGE = "https://magichub.com/datasets/thai-scripted-speech-corpus-daily-use-sentence/"
_LANGUAGES = ["tha"]
_LICENSE = Licenses.CC_BY_NC_ND_4_0.value
_LOCAL = False
_URLS = {
    _DATASETNAME: "https://magichub.com/df/df.php?file_name=Thai_Scripted_Speech_Corpus_Daily_Use_Sentence.zip",
}
_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ASRSTIDuSCDataset(datasets.GeneratorBasedBuilder):
    """ASR-STIDuSC consists transcribed Thai scripted speech focusing on daily use sentences"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "sptext"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "channel": datasets.Value("string"),
                    "uttrans_id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "transcription": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "speaker_gender": datasets.Value("string"),
                    "speaker_age": datasets.Value("int64"),
                    "speaker_region": datasets.Value("string"),
                    "speaker_device": datasets.Value("string"),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        data_paths = {
            _DATASETNAME: Path(dl_manager.download_and_extract(_URLS[_DATASETNAME])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_paths[_DATASETNAME],
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        # read UTTRANSINFO file
        # columns: channel, uttrans_id, speaker_id, prompt (empty field), transcription
        uttransinfo_filepath = os.path.join(filepath, "UTTRANSINFO.txt")
        with open(uttransinfo_filepath, "r", encoding="utf-8") as uttransinfo_file:
            uttransinfo_data = uttransinfo_file.readlines()
        uttransinfo_data = uttransinfo_data[1:]  # remove header
        uttransinfo_data = [s.strip("\n").split("\t") for s in uttransinfo_data]

        # read SPKINFO file
        # columns: channel, speaker_id, gender, age, region, device
        spkinfo_filepath = os.path.join(filepath, "SPKINFO.txt")
        with open(spkinfo_filepath, "r", encoding="utf-8") as spkinfo_file:
            spkinfo_data = spkinfo_file.readlines()
        spkinfo_data = spkinfo_data[1:]  # remove header
        spkinfo_data = [s.strip("\n").split("\t") for s in spkinfo_data]
        for i, s in enumerate(spkinfo_data):
            if s[2] == "M":
                s[2] = "male"
            elif s[2] == "F":
                s[2] = "female"
            else:
                s[2] = None
        # dictionary of metadata of each speaker
        spkinfo_dict = {s[1]: {"speaker_gender": s[2], "speaker_age": int(s[3]), "speaker_region": s[4], "speaker_device": s[5]} for s in spkinfo_data}

        for i, sample in enumerate(uttransinfo_data):
            wav_path = os.path.join(filepath, "WAV", sample[2], sample[1])

            if self.config.schema == "source":
                example = {
                    "id": str(i),
                    "channel": sample[0],
                    "uttrans_id": sample[1],
                    "speaker_id": sample[2],
                    "transcription": sample[4],
                    "path": wav_path,
                    "audio": wav_path,
                    "speaker_gender": spkinfo_dict[sample[2]]["speaker_gender"],
                    "speaker_age": spkinfo_dict[sample[2]]["speaker_age"],
                    "speaker_region": spkinfo_dict[sample[2]]["speaker_region"],
                    "speaker_device": spkinfo_dict[sample[2]]["speaker_device"],
                }
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                example = {
                    "id": str(i),
                    "speaker_id": sample[2],
                    "path": wav_path,
                    "audio": wav_path,
                    "text": sample[4],
                    "metadata": {"speaker_age": spkinfo_dict[sample[2]]["speaker_age"], "speaker_gender": spkinfo_dict[sample[2]]["speaker_gender"]},
                }

            yield i, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@inproceedings{hernandez-devaraj-2019-phfakenews,
  author    = {Fernandez, Aaron Carl T. and Devaraj, Madhavi},
  title     = {Computing the Linguistic-Based Cues of Fake News in the Philippines Towards its Detection},
  booktitle = {Proceedings of the 9th International Conference on Web Intelligence, Mining and Semantics},
  publisher = {Association for Computing Machinery},
  year      = {2019},
  url       = {https://dl.acm.org/doi/abs/10.1145/3326467.3326490},
  doi       = {10.1145/3326467.3326490},
  pages     = {1-9},
}
"""

_LOCAL = False
_LANGUAGES = ["eng"]
_DATASETNAME = "ph_fake_news_corpus"
_DESCRIPTION = """
The Philippine Fake News Corpus consists of news headlines and content from various "credible" and "non-credible"
national news outlets. "Credible" sources were national broadsheets available in the National Library of the
Philippines, while "non-credible" sources were sources included in lists of websites with fake or unverified content
provided by government and private institutions.
"""

_HOMEPAGE = "https://github.com/aaroncarlfernandez/Philippine-Fake-News-Corpus"
_LICENSE = Licenses.UNKNOWN.value
_URL = "https://github.com/aaroncarlfernandez/Philippine-Fake-News-Corpus/raw/master/Philippine%20Fake%20News%20Corpus.zip/"

_SUPPORTED_TASKS = [Tasks.FACT_CHECKING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class PhilippineFakeNewsDataset(datasets.GeneratorBasedBuilder):
    """
    Dataset of English news articles from the Philippines manually annotated as "credible" or
    "non-credible" based on source.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        features = datasets.Features(
            {
                "Headline": datasets.Value("string"),
                "Content": datasets.Value("string"),
                "Authors": datasets.Value("string"),
                "Date": datasets.Value("string"),
                "URL": datasets.Value("string"),
                "Brand": datasets.Value("string"),
                "Label": datasets.Value("string"),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "Philippine Fake News Corpus.csv"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        df = pd.read_csv(filepath, index_col=None, header="infer", encoding="utf-8")
        for index, example in df.iterrows():
            yield index, example.to_dict()


from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import json
import os

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from zipfile import ZipFile

_CITATION = """\
@inproceedings{sakti-tcast-2008,
    title = "Development of {I}ndonesian Large Vocabulary Continuous Speech Recognition System within {A-STAR} Project",
    author = "Sakti, Sakriani and Kelana, Eka and Riza, Hammam and Sakai, Shinsuke and Markov, Konstantin and Nakamura, Satoshi",
    booktitle = "Proc. IJCNLP Workshop on Technologies and Corpora for Asia-Pacific Speech Translation (TCAST)",
    year = "2008",
    pages = "19--24"
    address = "Hyderabad, India"
}


@inproceedings{sakti-icslp-2004,
    title = "Indonesian Speech Recognition for Hearing and Speaking Impaired People",
    author = "Sakti, Sakriani and Hutagaol, Paulus and Arman, Arry Akhmad and Nakamura, Satoshi",
    booktitle = "Proc. International Conference on Spoken Language Processing (INTERSPEECH - ICSLP)",
    year = "2004",
    pages = "1037--1040"
    address = "Jeju Island, Korea"
}

@article{sakti-s2st-csl-2013,
    title = "{A-STAR}: Toward Tranlating Asian Spoken Languages",
    author = "Sakti, Sakriani and Paul, Michael and Finch, Andrew and Sakai, Shinsuke and Thang, Tat Vu, and Kimura, Noriyuki 
    and Hori, Chiori and Sumita, Eiichiro and Nakamura, Satoshi and Park, Jun and Wutiwiwatchai, Chai and Xu, Bo and Riza, Hammam 
    and Arora, Karunesh and Luong, Chi Mai and Li, Haizhou",
    journal = "Special issue on Speech-to-Speech Translation, Computer Speech and Language Journal",
    volume = "27",
    number ="2",
    pages = "509--527",
    year = "2013",
    publisher = "Elsevier"
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indspeech_teldialog_lvcsr"

_DESCRIPTION = """
INDspeech_TELDIALOG_LVCSR is one of the first Indonesian speech datasets for large vocabulary continuous speech recognition (LVCSR) based on telephon application. R&D Division of PT Telekomunikasi Indonesia developed the data in 2005-2006, in collaboration with Advanced Telecommunication Research Institute International (ATR) Japan, as the continuation of the Asia-Pacific Telecommunity (APT) project [Sakti et al., 2004]. It has also been successfully used for developing Indonesian LVCSR in the Asian speech translation advanced research (A-STAR) project [Sakti et al., 2013].
"""

_HOMEPAGE = "https://github.com/s-sakti/data_indsp_teldialog_lvcsr"

_LICENSE = "CC-BY-NC-SA 4.0"


URL_TEMPLATE = {
    "lst": "https://raw.githubusercontent.com/s-sakti/data_indsp_teldialog_lvcsr/main/lst/",  # transcript.lst
    "speech": "https://github.com/s-sakti/data_indsp_teldialog_lvcsr/raw/main/speech/",  # Ind3/Ind304.zip~Ind400.zip
    "text": "https://github.com/s-sakti/data_indsp_teldialog_lvcsr/raw/main/text/",  # all_transcript.zip
}

_URLS = {
    "lst_spk_Ind": [URL_TEMPLATE["lst"] + "spk_Ind" + str(n) + ".lst" for n in range(0, 4)],
    "lst_spk_all": URL_TEMPLATE["lst"] + "spk_all.lst",
    "lst_spk_test": URL_TEMPLATE["lst"] + "spk_test.lst",
    "lst_spk_train": URL_TEMPLATE["lst"] + "spk_train.lst",
    "lst_transcript": URL_TEMPLATE["lst"] + "transcript.lst",
    "speech_Ind": [URL_TEMPLATE["speech"] + "Ind" + str(n) + "/Ind" + str(p).zfill(3) + ".zip" for n in range(0, 4) for p in range(n * 100 + 1, n * 100 + 101)],
    "transcript_all": URL_TEMPLATE["text"] + "all_transcript.zip",
    "transcript_spk": URL_TEMPLATE["text"] + "spk_transcript.zip",
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndSpeechTelDialLVCSR(datasets.GeneratorBasedBuilder):
    """INDspeech_TELDIALOG_LVCSR is one of the first Indonesian speech datasets for large vocabulary continuous speech recognition (LVCSR) based on telephon application. R&D Division of PT Telekomunikasi Indonesia developed the data in 2005-2006, in collaboration with Advanced Telecommunication Research Institute International (ATR) Japan, as the continuation of the Asia-Pacific Telecommunity (APT) project [Sakti et al., 2004]. It has also been successfully used for developing Indonesian LVCSR in the Asian speech translation advanced research (A-STAR) project [Sakti et al., 2013]."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"indspeech_teldialog_lvcsr_source",
            version=_SOURCE_VERSION,
            description="indspeech_teldialog_lvcsr source schema",
            schema="source",
            subset_id=f"indspeech_teldialog_lvcsr"
        ),
        SEACrowdConfig(
            name=f"indspeech_teldialog_lvcsr_seacrowd_sptext",
            version=_SOURCE_VERSION,
            description="indspeech_teldialog_lvcsr Nusantara schema",
            schema="seacrowd_sptext",
            subset_id=f"indspeech_teldialog_lvcsr"
        ),]
            
    DEFAULT_CONFIG_NAME = "indspeech_teldialog_lvcsr_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":

            features = datasets.Features(
               {
                   "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
               }
            )

        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[datasets.AutomaticSpeechRecognition(audio_column="audio", transcription_column="text")],
        )

    
    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        
        audio_files_dir = []
        for aud_url in _URLS["speech_Ind"]:
            onespeaker_folder = dl_manager.download_and_extract(aud_url)
            audio_files_dir.append(Path(os.path.join(onespeaker_folder, aud_url.split("/")[-1][:-4])))
            
        text_path = Path(dl_manager.download_and_extract(_URLS["lst_transcript"]))
        speak_list = Path(dl_manager.download_and_extract(_URLS["lst_spk_all"]))
        train_list = Path(dl_manager.download_and_extract(_URLS["lst_spk_train"]))
        test_list = Path(dl_manager.download_and_extract(_URLS["lst_spk_test"]))
        
        
          
        speaker_num2id = {}
        with open(speak_list) as f:
            for l in f.readlines():
                l = l.strip()
                speaker_num2id.update({l.split("_")[0]: l})
         
            
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,

                gen_kwargs={
                    "audio_files_dir": audio_files_dir,
                    "text_path": text_path,
                    "split": "train",
                    "file_list": train_list,
                    "speaker_num2id": speaker_num2id
                },
            ),
            
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "audio_files_dir": audio_files_dir,
                    "text_path": text_path,
                    "split": "test",
                    "file_list": test_list,
                    "speaker_num2id": speaker_num2id
                },
            )
        ]
    

    def _generate_examples(self, audio_files_dir: List, text_path: Path, split: str, file_list: Path, speaker_num2id: Dict) -> Tuple[int, Dict]:
        speaker_nums = []
        with open(file_list) as f:
            for l in f.readlines():
                speaker_nums.append(l.strip())
                
                
        sentid = {}
        with open(text_path) as f:
            for i, l in enumerate(f.readlines()):
                sentid.update({"appl_"+"%04d" % i: l.strip()})
                
                
        for wav_one_speaker_folder in audio_files_dir: #XXXX/Ind0/Ind001
            if wav_one_speaker_folder.name in speaker_nums:
                speaker_num = wav_one_speaker_folder.name #Ind001
                speaker_id = speaker_num2id[speaker_num] #Ind001_F_B

                for wave_file in os.listdir(wav_one_speaker_folder):
                    audio_id = wave_file[:-4]
                    sentence_id = "appl_"+wave_file[:-4].split('_')[-1]
                    text = sentid[sentence_id]
                    wav_path = os.path.join(wav_one_speaker_folder, wave_file)

                    if self.config.schema == "source":
                        ex = {
                            "id": audio_id,
                            "speaker_id": speaker_id,
                            "path": wav_path,
                            "audio": wav_path,
                            "text": text,
                        }
                        yield audio_id, ex

                    elif self.config.schema == "seacrowd_sptext":
                        ex = {
                            "id": audio_id,
                            "speaker_id": speaker_id,
                            "path": wav_path,
                            "audio": wav_path,
                            "text": text,
                            "metadata": {
                                "speaker_age": None,
                                "speaker_gender": speaker_id.split("_")[1],
                            }
                        }
                        yield audio_id, ex



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Dict, List, Tuple

import datasets

from seacrowd.sea_datasets.mtop_intent_classification.labels import (
    DOMAIN_LABELS, INTENT_LABELS)
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{li-etal-2021-mtop,
  author    = {Li, Haoran and Arora, Abhinav and Chen, Shuochi and Gupta, Anchit and Gupta, Sonal and Mehdad, Yashar},
  title     = {MTOP: A Comprehensive Multilingual Task-Oriented Semantic Parsing Benchmark},
  booktitle   = {Proceedings of the 16th Conference of the European Chapter of the Association for Computational Linguistics: Main Volume},
  publisher = {Association for Computational Linguistics},
  year      = {2021},
  url       = {https://aclanthology.org/2021.eacl-main.257},
  doi       = {10.18653/v1/2021.eacl-main.257},
  pages    = {2950-2962},
}
"""
_LOCAL = False
_LANGUAGES = ["tha"]
_DATASETNAME = "mtop_intent_classification"
_DESCRIPTION = """
This dataset contains annotated utterances from 6 languages, including Thai,
for semantic parsing. Queries corresponding to the chosen domains are crowdsourced.
 Two subsets are included in this dataset: 'domain' (eg. 'news', 'people', 'weather')
 and 'intent' (eg. 'GET_MESSAGE', 'STOP_MUSIC', 'END_CALL')
"""

_HOMEPAGE = "https://huggingface.co/mteb"
_LICENSE = Licenses.CC_BY_SA_4_0.value  # Found in original dataset (not HF) linked in paper
_URL = "https://huggingface.co/datasets/mteb/"


_SUPPORTED_TASKS = [Tasks.INTENT_CLASSIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class MTOPIntentClassificationDataset(datasets.GeneratorBasedBuilder):
    """Dataset of Thai sentences and their domains or intents."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SUBSETS = ["domain", "intent"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema for {subset} subset",
            schema="source",
            subset_id=f"{_DATASETNAME}_{subset}",
        )
        for subset in SUBSETS
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema for {subset} subset",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}_{subset}",
        )
        for subset in SUBSETS
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_domain_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("int64"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("int32"),
                    "label_text": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_text":
            if self.config.subset_id == "domain":
                labels = DOMAIN_LABELS
            elif self.config.subset_id == "intent":
                labels = INTENT_LABELS
            else:
                raise ValueError(f"Received unexpected schema name {self.config.name}")
            features = schemas.text_features(label_names=labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        # dl_manager not used since dataloader uses HF `load_dataset`
        return [datasets.SplitGenerator(name=split, gen_kwargs={"split": split._name}) for split in (datasets.Split.TRAIN, datasets.Split.VALIDATION, datasets.Split.TEST)]

    def _load_hf_data_from_remote(self, split: str) -> datasets.DatasetDict:
        """Load dataset from HuggingFace."""
        if self.config.subset_id not in ("domain", "intent"):
            raise ValueError(f"Received unexpected schema name {self.config.name}")
        HF_REMOTE_REF = "/".join(_URL.split("/")[-2:]) + f"mtop_{self.config.subset_id}"
        _hf_dataset_source = datasets.load_dataset(HF_REMOTE_REF, "th", split=split)
        return _hf_dataset_source

    def _generate_examples(self, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        data = self._load_hf_data_from_remote(split=split)
        for index, row in enumerate(data):
            if self.config.schema == "source":
                example = row

            elif self.config.schema == "seacrowd_text":
                example = {"id": str(index), "text": row["text"], "label": row["label_text"]}
            yield index, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME, Tasks

_LANGUAGES = ["ind", "eng"]
_CITATION = """\

@article{wang2020covost,
  title={Covost 2 and massively multilingual speech-to-text translation},
  author={Wang, Changhan and Wu, Anne and Pino, Juan},
  journal={arXiv preprint arXiv:2007.10310},
  year={2020}
}

@inproceedings{wang21s_interspeech,
  author={Wang, Changhan and Wu, Anne and Pino, Juan},
  title={{CoVoST 2 and Massively Multilingual Speech Translation}},
  year=2021,
  booktitle={Proc. Interspeech 2021},
  pages={2247--2251},
  url={https://www.isca-speech.org/archive/interspeech_2021/wang21s_interspeech}
  doi={10.21437/Interspeech.2021-2027}
}
"""

_DATASETNAME = "covost2"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_DESCRIPTION = """\
CoVoST2 is a large-scale multilingual speech translation corpus covering translations from 21 languages to English
and from English into 15 languages. The dataset is created using Mozilla's open-source Common Voice database of
crowdsourced voice recordings. There are 2,900 hours of speech represented in the corpus.
"""

_HOMEPAGE = "https://huggingface.co/datasets/covost2"

_LOCAL = False
_LICENSE = "CC BY-NC 4.0"

COMMONVOICE_URL_TEMPLATE = "https://voice-prod-bundler-ee1969a6ce8178826482b88e843c335139bd3fb4.s3.amazonaws.com/cv-corpus-6.1-2020-12-11/{lang}.tar.gz"
LANG_CODE = {"eng": "en", "ind": "id"}
LANG_COMBINATION_CODE = [("ind", "eng"), ("eng", "ind")]
_URLS = {_DATASETNAME: {"ind": COMMONVOICE_URL_TEMPLATE.format(lang=LANG_CODE["ind"]), "eng": COMMONVOICE_URL_TEMPLATE.format(lang=LANG_CODE["eng"])}}

_SUPPORTED_TASKS = [Tasks.SPEECH_TO_TEXT_TRANSLATION, Tasks.MACHINE_TRANSLATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(src_lang, tgt_lang, schema, version):
    if src_lang == "" or tgt_lang == "":
        raise ValueError(f"Invalid src_lang {src_lang} or tgt_lang {tgt_lang}")

    if schema not in ["source", "seacrowd_sptext", "seacrowd_t2t"]:
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="covost2_{src}_{tgt}_{schema}".format(src=src_lang, tgt=tgt_lang, schema=schema),
        version=datasets.Version(version),
        description="covost2 source schema for {schema} from {src} to {tgt}".format(schema=schema, src=src_lang, tgt=tgt_lang),
        schema=schema,
        subset_id="co_vo_st2_{src}_{tgt}".format(src=src_lang, tgt=tgt_lang),
    )


class Covost2(datasets.GeneratorBasedBuilder):
    """CoVoST2 dataset is a dataset mainly for speech to text translation task. The data was taken from Mozilla Common
    Voices dataset. In the implementation of the source schema, the audio and transcriptions of the source language,
    as well as the translated transcriptions are provided. In the implementation of the seacrowd schema, only the audio of the source language and transcriptions of the
    target language are provided. The source and target languages available are eng->ind and ind -> eng respectively.
    In addition to the speech to text translation, this dataset (text only) can be used as a machine translation for
    eng->ind and ind->eng.

    Side note: the amount of data takes about 40GB for the English source data and 1GB for the Indonesian source data.
    """

    COVOST_URL_TEMPLATE = "https://dl.fbaipublicfiles.com/covost/covost_v2.{src_lang}_{tgt_lang}.tsv.tar.gz"

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [seacrowd_config_constructor(src, tgt, "source", _SOURCE_VERSION) for (src, tgt) in LANG_COMBINATION_CODE]
        + [seacrowd_config_constructor(src, tgt, "seacrowd_sptext", _SEACROWD_VERSION) for (src, tgt) in LANG_COMBINATION_CODE]
        + [seacrowd_config_constructor(src, tgt, "seacrowd_t2t", _SEACROWD_VERSION) for (src, tgt) in LANG_COMBINATION_CODE]
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_eng_ind_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {"client_id": datasets.Value("string"), "file": datasets.Value("string"), "audio": datasets.Audio(sampling_rate=16_000), "sentence": datasets.Value("string"), "translation": datasets.Value("string"), "id": datasets.Value("string")}
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[datasets.AutomaticSpeechRecognition(audio_column="audio", transcription_column="sentences")] if (self.config.schema == "seacrowd_sptext" or self.config.schema == "source") else None,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        name_split = self.config.name.split("_")
        src_lang, tgt_lang = name_split[1], name_split[2]

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls[src_lang])

        src_lang = LANG_CODE[src_lang]
        tgt_lang = LANG_CODE[tgt_lang]

        data_dir = data_dir + "/" + "/".join(["cv-corpus-6.1-2020-12-11", src_lang])

        covost_tsv_path = self.COVOST_URL_TEMPLATE.format(src_lang=src_lang, tgt_lang=tgt_lang)
        extracted_dir = dl_manager.download_and_extract(covost_tsv_path)

        covost_tsv_filename = "covost_v2.{src_lang}_{tgt_lang}.tsv"
        covost_tsv_dir = os.path.join(extracted_dir, covost_tsv_filename.format(src_lang=src_lang, tgt_lang=tgt_lang))
        cv_tsv_dir = os.path.join(data_dir, "validated.tsv")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "covost_tsv_path": covost_tsv_dir,
                    "cv_tsv_path": cv_tsv_dir,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir,
                    "covost_tsv_path": covost_tsv_dir,
                    "cv_tsv_path": cv_tsv_dir,
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir,
                    "covost_tsv_path": covost_tsv_dir,
                    "cv_tsv_path": cv_tsv_dir,
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, covost_tsv_path: Path, cv_tsv_path: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        name_split = self.config.name.split("_")
        src_lang, tgt_lang = name_split[1], name_split[2]

        covost_tsv = self._load_df_from_tsv(covost_tsv_path)
        cv_tsv = self._load_df_from_tsv(cv_tsv_path)

        df = pd.merge(
            left=cv_tsv[["path", "sentence", "client_id"]],
            right=covost_tsv[["path", "translation", "split"]],
            how="inner",
            on="path",
        )
        if split == "train":
            df = df[(df["split"] == "train") | (df["split"] == "train_covost")]
        else:
            df = df[df["split"] == split]

        for id, row in df.iterrows():
            if self.config.schema == "source":
                yield id, {
                    "id": row["path"].replace(".mp3", ""),
                    "client_id": row["client_id"],
                    "sentence": row["sentence"],
                    "translation": row["translation"],
                    "file": os.path.join(filepath, "clips", row["path"]),
                    "audio": os.path.join(filepath, "clips", row["path"]),
                }
            elif self.config.schema == "seacrowd_sptext":
                yield id, {
                    "id": row["path"].replace(".mp3", ""),
                    "speaker_id": row["client_id"],
                    "text": row["translation"],
                    "path": os.path.join(filepath, "clips", row["path"]),
                    "audio": os.path.join(filepath, "clips", row["path"]),
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": None,
                    },
                }
            elif self.config.schema == "seacrowd_t2t":
                yield id, {"id": row["path"].replace(".mp3", ""), "text_1": row["sentence"], "text_2": row["translation"], "text_1_name": src_lang, "text_2_name": tgt_lang}
            else:
                raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

    @staticmethod
    def _load_df_from_tsv(path):
        return pd.read_csv(
            path,
            sep="\t",
            header=0,
            encoding="utf-8",
            escapechar="\\",
            quoting=csv.QUOTE_NONE,
            na_filter=False,
        )



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
import os
import re
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from translate.storage.tmx import tmxfile

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{anastasopoulos-etal-2020-tico,
    title = "{TICO}-19: the Translation Initiative for {CO}vid-19",
    author = {Anastasopoulos, Antonios  and
      Cattelan, Alessandro  and
      Dou, Zi-Yi  and
      Federico, Marcello  and
      Federmann, Christian  and
      Genzel, Dmitriy  and
      Guzm{\'a}n, Franscisco  and
      Hu, Junjie  and
      Hughes, Macduff  and
      Koehn, Philipp  and
      Lazar, Rosie  and
      Lewis, Will  and
      Neubig, Graham  and
      Niu, Mengmeng  and
      {\"O}ktem, Alp  and
      Paquin, Eric  and
      Tang, Grace  and
      Tur, Sylwia},
    booktitle = "Proceedings of the 1st Workshop on {NLP} for {COVID}-19 (Part 2) at {EMNLP} 2020",
    month = dec,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2020.nlpcovid19-2.5",
    doi = "10.18653/v1/2020.nlpcovid19-2.5",
}
"""

# We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LANGUAGES = ["ind", "ara", "spa", "fra", "hin", "por", "rus", "zho", "eng", "khm", "zlm", "mya", "tgl", "tam"]
_LOCAL = False
_SUPPORTED_LANG_PAIRS = [
    ("ind", "ara"),
    ("ind", "spa"),
    ("ind", "fra"),
    ("ind", "hin"),
    ("ind", "por"),
    ("ind", "rus"),
    ("ind", "zho"),
    ("ind", "eng"),
    ("ara", "ind"),
    ("spa", "ind"),
    ("fra", "ind"),
    ("hin", "ind"),
    ("por", "ind"),
    ("rus", "ind"),
    ("zho", "ind"),
    ("eng", "ind"),
    ("khm", "eng"),
    ("eng", "khm"),
    ("mya", "eng"),
    ("eng", "mya"),
    ("zlm", "eng"),
    ("eng", "zlm"),
    ("tgl", "eng"),
    ("eng", "tgl"),
    ("tam", "eng"),
    ("eng", "tam"),
]

_LANG_CODE_MAP = {"ind": "id", "ara": "ar", "spa": "es-LA", "fra": "fr", "hin": "hi", "por": "pt-BR", "rus": "ru", "zho": "zh", "eng": "en", "khm": "km", "zlm": "ms", "mya": "my", "tgl": "tl", "tam": "ta"}

_DEVTEST_LANG_PAIRS = [_LANG_CODE_MAP[source_lang] + "-" + _LANG_CODE_MAP[target_lang] for (source_lang, target_lang) in _SUPPORTED_LANG_PAIRS if (source_lang == "eng" or target_lang == "eng")]

_DATASETNAME = "tico_19"

_DESCRIPTION = """\
TICO-19 (Translation Initiative for COVID-19) is sampled from a variety of public sources containing
COVID-19 related content, representing different domains (e.g., news, wiki articles, and others). TICO-19
includes 30 documents (3071 sentences, 69.7k words) translated from English into 36 languages: Amharic,
Arabic (Modern Standard), Bengali, Chinese (Simplified), Dari, Dinka, Farsi, French (European), Hausa,
Hindi, Indonesian, Kanuri, Khmer (Central), Kinyarwanda, Kurdish Kurmanji, Kurdish Sorani, Lingala,
Luganda, Malay, Marathi, Myanmar, Nepali, Nigerian Fulfulde, Nuer, Oromo, Pashto, Portuguese (Brazilian),
Russian, Somali, Spanish (Latin American), Swahili, Congolese Swahili, Tagalog, Tamil, Tigrinya, Urdu, Zulu.
"""

_HOMEPAGE = "https://tico-19.github.io"

_LICENSE = "CC0"

_URLS = {"evaluation": "https://tico-19.github.io/data/tico19-testset.zip", "all": "https://tico-19.github.io/data/TM/all.{lang_pairs}.tmx.zip"}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(lang_source, lang_target, schema, version):
    """Construct SEACrowdConfig with tico_19_{lang_source}_{lang_target}_{schema} as the name format"""
    if schema != "source" and schema != "seacrowd_t2t":
        raise ValueError(f"Invalid schema: {schema}")

    if lang_source == "" and lang_target == "":
        return SEACrowdConfig(
            name="tico_19_{schema}".format(schema=schema),
            version=datasets.Version(version),
            description="tico_19 {schema} schema for default language pair (eng-ind)".format(schema=schema),
            schema=schema,
            subset_id="tico_19",
        )
    else:
        return SEACrowdConfig(
            name="tico_19_{src}_{tgt}_{schema}".format(src=lang_source, tgt=lang_target, schema=schema),
            version=datasets.Version(version),
            description="tico_19 {schema} schema for {src}-{tgt} language pair".format(src=lang_source, tgt=lang_target, schema=schema),
            schema=schema,
            subset_id="tico_19",
        )


class Tico19(datasets.GeneratorBasedBuilder):
    """TICO-19 is MT dataset sampled from a variety of public sources containing COVID-19 related content"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [seacrowd_config_constructor(src, tgt, schema, version) for src, tgt in [("", "")] + _SUPPORTED_LANG_PAIRS for schema, version in zip(["source", "seacrowd_t2t"], [_SOURCE_VERSION, _SEACROWD_VERSION])]

    DEFAULT_CONFIG_NAME = "tico_19_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "sourceLang": datasets.Value("string"),
                    "targetLang": datasets.Value("string"),
                    "sourceString": datasets.Value("string"),
                    "targetString": datasets.Value("string"),
                    "stringID": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "license": datasets.Value("string"),
                    "translatorId": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        try:
            lang_pairs_config = re.search("tico_19_(.+?)_(source|seacrowd_t2t)", self.config.name).group(1)
            lang_src, lang_tgt = lang_pairs_config.split("_")
        except AttributeError:
            lang_src, lang_tgt = "eng", "ind"

        lang_pairs = _LANG_CODE_MAP[lang_src] + "-" + _LANG_CODE_MAP[lang_tgt]

        # dev & test split only applicable to eng-[sea language] language pair
        if lang_pairs in set(_DEVTEST_LANG_PAIRS):
            lang_sea = _LANG_CODE_MAP[lang_tgt] if lang_src == "eng" else _LANG_CODE_MAP[lang_src]

            data_dir = dl_manager.download_and_extract(_URLS["evaluation"])
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={"filepath": os.path.join(data_dir, "tico19-testset", "test", f"test.en-{lang_sea}.tsv"), "lang_source": lang_src, "lang_target": lang_tgt},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={"filepath": os.path.join(data_dir, "tico19-testset", "dev", f"dev.en-{lang_sea}.tsv"), "lang_source": lang_src, "lang_target": lang_tgt},
                ),
            ]
        else:
            data_dir = dl_manager.download_and_extract(_URLS["all"].format(lang_pairs=lang_pairs))
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={"filepath": os.path.join(data_dir, f"all.{lang_pairs}.tmx"), "lang_source": lang_src, "lang_target": lang_tgt},
                )
            ]

    def _generate_examples(self, filepath: Path, lang_source: str, lang_target: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            # eng-[sea language] language pair dataset provided in .tsv format
            if f"{_LANG_CODE_MAP[lang_source]}-{_LANG_CODE_MAP[lang_target]}" in set(_DEVTEST_LANG_PAIRS):
                with open(filepath, encoding="utf-8") as f:
                    reader = csv.reader(f, delimiter="\t", quotechar='"')
                    for id_, row in enumerate(reader):
                        if id_ == 0:
                            continue
                        if lang_source == "eng":
                            source_lang = row[0]
                            target_lang = row[1]
                            source_string = row[2]
                            target_string = row[3]
                        else:
                            source_lang = row[1]
                            target_lang = row[0]
                            source_string = row[3]
                            target_string = row[2]
                        yield id_, {
                            "sourceLang": source_lang,
                            "targetLang": target_lang,
                            "sourceString": source_string,
                            "targetString": target_string,
                            "stringID": row[4],
                            "url": row[5],
                            "license": row[6],
                            "translatorId": row[7],
                        }

            # all language pairs except eng-ind dataset provided in .tmx format
            else:
                with open(filepath, "rb") as f:
                    tmx_file = tmxfile(f)

                for id_, node in enumerate(tmx_file.unit_iter()):
                    try:
                        url = [text for text in node.xmlelement.itertext("prop")][0]
                    except Exception:
                        url = ""
                    yield id_, {
                        "sourceLang": _LANG_CODE_MAP[lang_source],
                        "targetLang": _LANG_CODE_MAP[lang_target],
                        "sourceString": node.source,
                        "targetString": node.target,
                        "stringID": node.getid(),
                        "url": url,
                        "license": "",
                        "translatorId": "",
                    }

        elif self.config.schema == "seacrowd_t2t":
            if f"{_LANG_CODE_MAP[lang_source]}-{_LANG_CODE_MAP[lang_target]}" in set(_DEVTEST_LANG_PAIRS):
                with open(filepath, encoding="utf-8") as f:
                    reader = csv.reader(f, delimiter="\t", quotechar='"')
                    for id_, row in enumerate(reader):
                        if id_ == 0:
                            continue
                        if lang_source == "eng":
                            source_string = row[2]
                            target_string = row[3]
                        else:
                            source_string = row[3]
                            target_string = row[2]
                        yield id_, {"id": row[4], "text_1": source_string, "text_2": target_string, "text_1_name": lang_source, "text_2_name": lang_target}
            else:
                with open(filepath, "rb") as f:
                    tmx_file = tmxfile(f)

                for id_, node in enumerate(tmx_file.unit_iter()):
                    yield id_, {"id": node.getid(), "text_1": node.source, "text_2": node.target, "text_1_name": lang_source, "text_2_name": lang_target}



"""
SEA Crowd Data Loader for Bloom VIST.
"""
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = r"""
@inproceedings{leong-etal-2022-bloom,
    title = "Bloom Library: Multimodal Datasets in 300+ Languages for a Variety of Downstream Tasks",
    author = "Leong, Colin  and
      Nemecek, Joshua  and
      Mansdorfer, Jacob  and
      Filighera, Anna  and
      Owodunni, Abraham  and
      Whitenack, Daniel",
    editor = "Goldberg, Yoav  and
      Kozareva, Zornitsa  and
      Zhang, Yue",
    booktitle = "Proceedings of the 2022 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2022",
    address = "Abu Dhabi, United Arab Emirates",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.emnlp-main.590",
    doi = "10.18653/v1/2022.emnlp-main.590",
    pages = "8608--8621",
}
"""

logger = datasets.logging.get_logger(__name__)

# this config is created for SEACrowd Dataloader
_LANG_CONFIG = {
    "abc": "Ambala Ayta",
    "ahk": "Akha",
    "bfn": "Bunak",
    "bjn": "Banjar",
    "bkx": "Baikeno",
    "brb": "Brao",
    "brv": "Western Bru",
    "bya": "Batak",
    "bzi": "Bisu",
    "ceb": "Cebuano",
    "cgc": "Kagayanen",
    "cmo": "Central Mnong",
    "ddg": "Fataluku",
    "dmg": "Upper Kinabatangan",
    "dnw": "Western Dani",
    "dtp": "Kadazan Dusun",
    "enc": "En",
    "fil": "Filipino",
    "hil": "Hiligaynon",
    "hro": "Haroi",
    "idt": "Idaté",
    "ilo": "Ilocano",
    "ind": "Indonesian",
    "jra": "Jarai",
    "kak": "Kalanguya",
    "khb": "Lü",
    "khm": "Khmer",
    "kqr": "Kimaragang",
    "krr": "Krung",
    "ksw": "S’gaw Karen",
    "lhu": "Lahu",
    "lsi": "Lacid",
    "lwl": "Eastern Lawa",
    "mdr": "Mandar",
    "mgm": "Mambae",
    "mhx": "Lhao Vo",
    "mkz": "Makasae",
    "mry": "Mandaya",
    "msb": "Masbatenyo",
    "mya": "Burmese",
    "nod": "Northern Thai",
    "nxa": "Nauete",
    "nxl": "South Nuaulu",
    "pag": "Pangasinan",
    "pce": "Ruching Palaung",
    "pea": "Peranakan Indonesian",
    "pmf": "Pamona",
    "psp": "Filipino Sign Language",
    "sea": "Semai",
    "sgd": "Surigaonon",
    "sml": "Central Sama",
    "snl": "Sangil",
    "tdt": "Tetun Dili",
    "tet": "Tetun",
    "tha": "Thai",
    "tkd": "Tukudede",
    "tpu": "Tampuan",
    "war": "Waray-Waray",
    "wms": "Wambon",
    "yet": "Yetfa",
    "yin": "Riang Lai",
    "zlm": "Malay",
}

_LOCAL = False
_LANGUAGES = list(_LANG_CONFIG.keys())


_DATASETNAME = "bloom_vist"
_DESCRIPTION = r"""
BLOOM VIST is a visual storytelling of books that consists of 62 languages indigenous to SEA.
This dataset is owned by Bloom, a free, open-source software developed by SIL International and associated with Bloom Library, app, and services.
This dataset is released with the LICENSE family of Creative Commons (although each story datapoints has its licensing in more detail,
e.g cc-by, cc-by-nc, cc-by-nd, cc-by-sa, cc-by-nc-nd, cc-by-nc-sa).
Before using this dataloader, please accept the acknowledgement at https://huggingface.co/datasets/sil-ai/bloom-vist and use huggingface-cli login for authentication.
"""

_HOMEPAGE = "https://huggingface.co/datasets/sil-ai/bloom-vist"
_LICENSE = Licenses.CC.value

_URL = "https://huggingface.co/datasets/sil-ai/bloom-vist"
_HF_REMOTE_REF = "/".join(_URL.split("/")[-2:])

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]
_SOURCE_VERSION = "0.1.0"
_SEACROWD_VERSION = "1.0.0"

CONFIG_SUFFIXES_FOR_TASK = [TASK_TO_SCHEMA.get(task).lower() for task in _SUPPORTED_TASKS]


def conform_init_config():
    """Assertion Function for Instantiated Configs"""
    if len(_LANGUAGES) == 0:
        raise AssertionError("No Languages detected from config!")
    if len(CONFIG_SUFFIXES_FOR_TASK) != len(_SUPPORTED_TASKS):
        raise AssertionError("Config prefixes don't matched in terms of `len` with `_SUPPORTED_TASKS`!")
    if len(CONFIG_SUFFIXES_FOR_TASK) == 0:
        raise AssertionError("Config prefixes and `_SUPPORTED_TASKS` have `len` of 0!")


conform_init_config()


def construct_configs_on_langs(languages: list = None) -> List[SEACrowdConfig]:
    """
    The function `construct_configs` constructs a list of SEACrowdConfig objects based on the provided
    languages or a default language, and returns the list.

    input:
        languages (list, default None): The `languages` parameter is a list that specifies the languages for which the
        configurations need to be constructed. If no languages are provided (value=None), the first value in language config
        will be used.
    output:
        a list of `SEACrowdConfig` objects based on instantiated init variables
    """

    # set output var
    config_list = []

    # construct zipped arg for config instantiation
    TASKS_AND_CONFIG_SUFFIX_PAIRS = list(zip(_SUPPORTED_TASKS, CONFIG_SUFFIXES_FOR_TASK))

    # implement source schema
    version, config_name_prefix = _SOURCE_VERSION, "source"
    config_list += [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}",
            version=datasets.Version(version),
            description=f"{_DATASETNAME} {config_name_prefix} schema for language code {_LANG}",
            schema=f"{config_name_prefix}",
            subset_id=_LANG,
        )
        for _LANG in languages
    ]

    # implement SEACrowd schema
    version, config_name_prefix = _SEACROWD_VERSION, "seacrowd"
    for task_obj, config_name_suffix in TASKS_AND_CONFIG_SUFFIX_PAIRS:
        config_list += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}_{config_name_suffix}",
                version=datasets.Version(version),
                description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name} and language code {_LANG}",
                schema=f"{config_name_prefix}_{config_name_suffix}",
                subset_id=_LANG,
            )
            for _LANG in languages
        ]
    return config_list


class BloomVISTDataset(datasets.GeneratorBasedBuilder):
    """Bloom VIST dataset, subsetted from https://huggingface.co/datasets/sil-ai/bloom-vist"""

    # get all schema w/o lang arg + get all schema w/ lang arg
    BUILDER_CONFIGS = construct_configs_on_langs(_LANGUAGES)

    def _info(self) -> datasets.DatasetInfo:
        _config_schema_name = self.config.schema
        logger.info(f"Received schema name: {self.config.schema}")
        # source schema
        if _config_schema_name == "source":
            features = datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "license": datasets.Value("string"),
                    "album_id": datasets.Value("string"),
                    "story": datasets.Sequence(
                        feature={"image_id": datasets.Value("string"), "image_url": datasets.Value("string"), "story_index": datasets.Value("int32"), "story_id": datasets.Value("string"), "text": datasets.Value("string")}, length=-1, id=None
                    ),
                }
            )

        # image-text schema
        elif _config_schema_name == "seacrowd_imtext":
            features = schemas.image_text_features()

        else:
            raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        hf_dset_dict = datasets.load_dataset(_HF_REMOTE_REF, self.config.subset_id)

        return [datasets.SplitGenerator(name=datasets.Split(dset_key), gen_kwargs={"hf_dset": dset}) for dset_key, dset in hf_dset_dict.items() if dset.num_rows > 0]

    def _generate_examples(self, hf_dset) -> Tuple[int, Dict]:
        _config_schema_name = self.config.schema

        _idx = 0
        for datapoints in hf_dset:
            # for source schema, the `_idx` will be taken from "album_id" value
            if _config_schema_name == "source":
                yield datapoints["album_id"], {colname: datapoints[colname] for colname in self.info.features}

            # for seacrowd schema, the `_idx` will be created manually
            # since one album_id has multiple pairs of image-text
            elif _config_schema_name == "seacrowd_imtext":
                # check the len of the features in sequenced columns
                # since in source hf there's no validation on data integrity
                _len_vars = []
                _ftrs_in_seq = ("image_id", "image_url", "story_index", "story_id", "text")
                story_data = datapoints["story"]
                for ftr in _ftrs_in_seq:
                    _len_vars.append(len(story_data[ftr]))

                # skip story w/ mismatched infos
                if max(_len_vars) != min(_len_vars):
                    continue

                for num_data in range(max(_len_vars)):
                    yield _idx, {"id": _idx, "image_paths": [story_data["image_url"][num_data]], "texts": story_data["text"][num_data], "metadata": {"context": datapoints["title"], "labels": []}}
                    _idx += 1

            else:
                raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")



"""
End-to-end Task Oriented Dialogue Schema

This schema will adapt the WoZ schema that has been used in several end-to-end task-oriented dialogue (TOD) task.
URL: https://huggingface.co/datasets/woz_dialogue
"""
import datasets

"""
EXAMPLE:

[
	{
		"turn_label": [ [ "food", "eritrean" ] ],
		"system_utterance": "",
		"turn_idx": 0,
		"belief_state": [ { "slots": [ [ "food", "eritrean" ] ], "act": "inform" } ],
		"user_utterance": "Are there any eritrean restaurants in town?",
		"system_acts": []
	},
	{
		"turn_label": [ [ "food", "chinese" ] ],
		"system_utterance": "No, there are no eritrean restaurants in town. Would you like a different restaurant? ",
		"turn_idx": 1,
		"belief_state": [ { "slots": [ [ "food", "chinese" ] ], "act": "inform" } ],
		"user_utterance": "How about Chinese food?",
		"system_acts": []
	},
	{
		"turn_label": [ [ "area", "east" ] ],
		"system_utterance": "There is a wide variety of Chinese restaurants, do you have an area preference or a price preference to narrow it down?",
		"turn_idx": 2,
		"belief_state": [ { "slots": [ [ "food", "chinese" ] ], "act": "inform" }, { "slots": [ [ "area", "east" ] ], "act": "inform" } ],
		"user_utterance": "I would like the East part of town.",
		"system_acts": [ [ "area" ] ]
	},
	{
		"turn_label": [ [ "request", "postcode" ], [ "request", "phone" ], [ "request", "address" ] ],
		"system_utterance": "Yu Garden is a chinese restaurant in the east area.",
		"turn_idx": 3,
		"belief_state": [ { "slots": [ [ "slot", "postcode" ] ], "act": "request" }, { "slots": [ [ "slot", "phone" ] ], "act": "request" }, { "slots": [ [ "slot", "address" ] ], "act": "request" }, { "slots": [ [ "food", "chinese" ] ], "act": "inform" }, { "slots": [ [ "area", "east" ] ], "act": "inform" } ],
		"user_utterance": "Could I get the address, phone number, and postcode of Yu Garden?",
		"system_acts": []
	},
	{
		"turn_label": [],
		"system_utterance": "Phone is 01223 248882, address and postcode are 529 Newmarket Road Fen Ditton C.B 5, 8 P.A",
		"turn_idx": 4,
		"belief_state": [ { "slots": [ [ "food", "chinese" ] ], "act": "inform" }, { "slots": [ [ "area", "east" ] ], "act": "inform" } ],
		"user_utterance": "Thank you. That is all the information I needed. Bye bye!",
		"system_acts": []}
]
"""

features = datasets.Features(
    {
        "dialogue_idx": datasets.Value("int32"),
        "dialogue": [
            {
                "turn_label": datasets.Sequence(datasets.Sequence(datasets.Value("string"))),
                "system_utterance": datasets.Value("string"),
                "turn_idx": datasets.Value("int32"),
                "belief_state": [
                    {
                        "slots": datasets.Sequence(datasets.Sequence(datasets.Value("string"))),
                        "act": datasets.Value("string"),
                    }
                ],
                "user_utterance": datasets.Value("string"),
                "system_acts": datasets.Sequence(datasets.Sequence(datasets.Value("string"))),
            }
        ],
    }
)



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@inproceedings{miranda-2023-developing,
    title = {Developing a Named Entity Recognition Dataset for Tagalog},
    author = "Miranda, Lester James Validad",
    booktitle = "Proceedings of the First Workshop for Southeast Asian Language Processing (SEALP),"
    month = nov,
    year = 2023,
    address = "Online",
    publisher = "Association for Computational Linguistics",
}
"""

_LOCAL = False
_LANGUAGES = ["tgl"]
_DATASETNAME = "tlunified_ner"
_DESCRIPTION = """\
This dataset contains the annotated TLUnified corpora from Cruz and Cheng
(2021). It is a curated sample of around 7,000 documents for the named entity
recognition (NER) task. The majority of the corpus are news reports in Tagalog,
resembling the domain of the original ConLL 2003. There are three entity types:
Person (PER), Organization (ORG), and Location (LOC).
"""

_HOMEPAGE = "https://huggingface.co/ljvmiranda921/tlunified-ner"
_LICENSE = Licenses.GPL_3_0.value
_URLS = {
    "train": "https://huggingface.co/datasets/ljvmiranda921/tlunified-ner/resolve/main/corpus/iob/train.iob",
    "dev": "https://huggingface.co/datasets/ljvmiranda921/tlunified-ner/resolve/main/corpus/iob/dev.iob",
    "test": "https://huggingface.co/datasets/ljvmiranda921/tlunified-ner/resolve/main/corpus/iob/test.iob",
}

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class TLUnifiedNERDataset(datasets.GeneratorBasedBuilder):
    """Tagalog Named Entity Recognition dataset from https://huggingface.co/ljvmiranda921/tlunified-ner"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "seq_label"
    LABEL_CLASSES = ["O", "B-PER", "I-PER", "B-ORG", "I-ORG", "B-LOC", "I-LOC"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(datasets.features.ClassLabel(names=self.LABEL_CLASSES)),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.seq_label_features(self.LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_files = {
            "train": Path(dl_manager.download_and_extract(_URLS["train"])),
            "dev": Path(dl_manager.download_and_extract(_URLS["dev"])),
            "test": Path(dl_manager.download_and_extract(_URLS["test"])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"], "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["dev"], "split": "dev"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"], "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        # The only difference between the source schema and the seacrowd seq_label schema is the dictionary keys.
        # The implementation is the same.
        label_key = "ner_tags" if self.config.schema == "source" else "labels"
        with open(filepath, encoding="utf-8") as f:
            guid = 0
            tokens = []
            ner_tags = []
            for line in f:
                if line.startswith("-DOCSTART-") or line == "" or line == "\n":
                    if tokens:
                        yield guid, {
                            "id": str(guid),
                            "tokens": tokens,
                            label_key: ner_tags,
                        }
                        guid += 1
                        tokens = []
                        ner_tags = []
                else:
                    # TLUnified-NER iob are separated by \t
                    token, ner_tag = line.split("\t")
                    tokens.append(token)
                    ner_tags.append(ner_tag.rstrip())
            # Last example
            if tokens:
                yield guid, {
                    "id": str(guid),
                    "tokens": tokens,
                    label_key: ner_tags,
                }



from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.sea_datasets.x_fact.utils.x_fact_utils import \
    load_x_fact_dataset
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{gupta2021xfact,
      title={{X-FACT: A New Benchmark Dataset for Multilingual Fact Checking}},
      author={Gupta, Ashim and Srikumar, Vivek},
      booktitle = "Proceedings of the 59th Annual Meeting of the Association for Computational Linguistics",
      month = jul,
      year = "2021",
      address = "Online",
      publisher = "Association for Computational Linguistics",
}
"""
_DATASETNAME = "x_fact"

_DESCRIPTION = """\
X-FACT: the largest publicly available multilingual dataset for factual verification of naturally existing realworld claims.
"""

_HOMEPAGE = "https://github.com/utahnlp/x-fact"

_LANGUAGES = [
    'ara', 'aze', 'ben', 'deu', 'spa', 
    'fas', 'fra', 'guj', 'hin', 'ind', 
    'ita', 'kat', 'mar', 'nor', 'nld', 
    'pan', 'pol', 'por', 'ron', 'rus',
    'sin', 'srp', 'sqi', 'tam', 'tur'
]
_LOCAL = False

_LICENSE = "MIT"

_URLS = {
    "train": "https://raw.githubusercontent.com/utahnlp/x-fact/main/data/x-fact-including-en/train.all.tsv",
    "validation": "https://raw.githubusercontent.com/utahnlp/x-fact/main/data/x-fact-including-en/dev.all.tsv",
    "test": {
        "in_domain": "https://raw.githubusercontent.com/utahnlp/x-fact/main/data/x-fact-including-en/test.all.tsv",
        "out_domain": "https://raw.githubusercontent.com/utahnlp/x-fact/main/data/x-fact-including-en/ood.tsv",
    },
}

_SUPPORTED_TASKS = [Tasks.FACT_CHECKING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class XFact(datasets.GeneratorBasedBuilder):
    """X-FACT: the largest publicly available multilingual dataset for factual verification of naturally existing realworld claims."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="x_fact_source",
            version=SOURCE_VERSION,
            description="x_fact source schema",
            schema="source",
            subset_id="x_fact",
        ),
    ]

    DEFAULT_CONFIG_NAME = "x_fact_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "language": datasets.Value("string"),
                    "site": datasets.Value("string"),
                    "evidence_1": datasets.Value("string"),
                    "evidence_2": datasets.Value("string"),
                    "evidence_3": datasets.Value("string"),
                    "evidence_4": datasets.Value("string"),
                    "evidence_5": datasets.Value("string"),
                    "link_1": datasets.Value("string"),
                    "link_2": datasets.Value("string"),
                    "link_3": datasets.Value("string"),
                    "link_4": datasets.Value("string"),
                    "link_5": datasets.Value("string"),
                    "claimDate": datasets.Value("string"),
                    "reviewDate": datasets.Value("string"),
                    "claimant": datasets.Value("string"),
                    "claim": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": _URLS["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": _URLS["validation"],
                    "split": "dev",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.splits.NamedSplit("TEST_IN_DOMAIN"),
                gen_kwargs={
                    "filepath": _URLS["test"]["in_domain"],
                    "split": "test_in_domain",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.splits.NamedSplit("TEST_OUT_DOMAIN"),
                gen_kwargs={
                    "filepath": _URLS["test"]["out_domain"],
                    "split": "test_out_domain",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:

        df = load_x_fact_dataset(filepath)
        if self.config.schema == "source":
            for row in df.itertuples():
                entry = {
                    "language": row.language,
                    "site": row.site,
                    "evidence_1": row.evidence_1,
                    "evidence_2": row.evidence_2,
                    "evidence_3": row.evidence_3,
                    "evidence_4": row.evidence_4,
                    "evidence_5": row.evidence_5,
                    "link_1": row.link_1,
                    "link_2": row.link_2,
                    "link_3": row.link_3,
                    "link_4": row.link_4,
                    "link_5": row.link_5,
                    "claimDate": row.claimDate,
                    "reviewDate": row.reviewDate,
                    "claimant": row.claimant,
                    "claim": row.claim,
                    "label": row.label,
                }
                yield row.index, entry



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import conllu
import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@INPROCEEDINGS{10053062,
  author={Samsuri, Mukhlizar Nirwan and Yuliawati, Arlisa and Alfina, Ika},
  booktitle={2022 5th International Seminar on Research of Information Technology and Intelligent Systems (ISRITI)},
  title={A Comparison of Distributed, PAM, and Trie Data Structure Dictionaries in Automatic Spelling Correction for Indonesian Formal Text},
  year={2022},
  pages={525-530},
  keywords={Seminars;Dictionaries;Data structures;Intelligent systems;Information technology;automatic spelling correction;distributed dictionary;non-word error;trie data structure;Partition Around Medoids},
  doi={10.1109/ISRITI56927.2022.10053062},
  url = {https://ieeexplore.ieee.org/document/10053062},
}
"""

_DATASETNAME = "etos"

_DESCRIPTION = """\
ETOS (Ejaan oTOmatiS) is a dataset for parts-of-speech (POS) tagging for formal Indonesian
text. It consists of 200 sentences, with 4,323 tokens in total, annotated following the
CoNLL format.
"""

_HOMEPAGE = "https://github.com/ir-nlp-csui/etos"

_LANGUAGES = ["ind"]

_LICENSE = Licenses.AGPL_3_0.value

_LOCAL = False

_URLS = "https://raw.githubusercontent.com/ir-nlp-csui/etos/main/gold_standard.conllu"

_SUPPORTED_TASKS = [Tasks.POS_TAGGING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class ETOSDataset(datasets.GeneratorBasedBuilder):
    """
    ETOS is an Indonesian parts-of-speech (POS) tagging dataset from https://github.com/ir-nlp-csui/etos.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    UPOS_TAGS = [
        "NOUN",
        "PUNCT",
        "ADP",
        "NUM",
        "SYM",
        "SCONJ",
        "ADJ",
        "PART",
        "DET",
        "CCONJ",
        "PROPN",
        "PRON",
        "X",
        "_",
        "ADV",
        "INTJ",
        "VERB",
        "AUX",
    ]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} sequence labeling schema",
            schema="seacrowd_seq_label",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "sent_id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "lemmas": datasets.Sequence(datasets.Value("string")),
                    "upos": datasets.Sequence(datasets.features.ClassLabel(names=self.UPOS_TAGS)),
                    "xpos": datasets.Sequence(datasets.Value("string")),
                    "feats": datasets.Sequence(datasets.Value("string")),
                    "head": datasets.Sequence(datasets.Value("string")),
                    "deprel": datasets.Sequence(datasets.Value("string")),
                    "deps": datasets.Sequence(datasets.Value("string")),
                    "misc": datasets.Sequence(datasets.Value("string")),
                }
            )

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.UPOS_TAGS)

        else:
            raise ValueError(f"Invalid schema: '{self.config.schema}'")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """
        Returns SplitGenerators.
        """

        train_path = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """
        Yields examples as (key, example) tuples.
        """

        with open(filepath, "r", encoding="utf-8") as data_file:
            tokenlist = list(conllu.parse_incr(data_file))

        for idx, sent in enumerate(tokenlist):
            if "sent_id" in sent.metadata:
                sent_id = sent.metadata["sent_id"]
            else:
                sent_id = idx

            tokens = [token["form"] for token in sent]

            if "text" in sent.metadata:
                txt = sent.metadata["text"]
            else:
                txt = " ".join(tokens)

            if self.config.schema == "source":
                yield idx, {
                    "sent_id": str(sent_id),
                    "text": txt,
                    "tokens": tokens,
                    "lemmas": [token["lemma"] for token in sent],
                    "upos": [token["upos"] for token in sent],
                    "xpos": [token["xpos"] for token in sent],
                    "feats": [str(token["feats"]) for token in sent],
                    "head": [str(token["head"]) for token in sent],
                    "deprel": [str(token["deprel"]) for token in sent],
                    "deps": [str(token["deps"]) for token in sent],
                    "misc": [str(token["misc"]) for token in sent],
                }

            elif self.config.schema == "seacrowd_seq_label":
                yield idx, {
                    "id": str(sent_id),
                    "tokens": tokens,
                    "labels": [token["upos"] for token in sent],
                }

            else:
                raise ValueError(f"Invalid schema: '{self.config.schema}'")
